// Logic simulation for the 74xx21 IC for the LogicSimulator
// Copyright (C) 2022 Sascha Nitsch

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// eslint-disable-next-line @typescript-eslint/no-unused-vars
class Logic74xx21 extends BaseComponent {
  private a = false;
  private b = false;
  private c = false;
  private d = false;
  private q = false;
  constructor(simulator: Simulator, id: string, param: BaseComponentParam) {
    super(simulator, id, param);
    this.pins.set('a', new TriState(this, -30, -15));
    this.pins.set('b', new TriState(this, -30, -5));
    this.pins.set('c', new TriState(this, -30, 5));
    this.pins.set('d', new TriState(this, -30, 15));
    this.pins.set('q', new TriState(this, 30, 0));
  }

  setup(canvas: SVGElement) {
    super.doSetup('logic74xx21', canvas);
  }

  io() {
    this.a = this.binary(this.getPin('a').getAndReset(), true);
    this.b = this.binary(this.getPin('b').getAndReset(), true);
    this.c = this.binary(this.getPin('c').getAndReset(), true);
    this.d = this.binary(this.getPin('d').getAndReset(), true);
  }

  update(): boolean {
    if (this.startdelay) {
      --this.startdelay;
      return false;
    }
    const oldstate = this.q;
    let mod = false;
    this.q = this.a && this && this.c && this.d;
    mod ||= this.q !== oldstate;
    this.getPin('q').setBool(this.q);
    return mod;
  }
}
