// Logic inverted Tri-State component for the LogicSimulator
// Copyright (C) 2022 Sascha Nitsch

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// eslint-disable-next-line @typescript-eslint/no-unused-vars
class LogicInvTristate extends BaseComponent {
  private a = false;
  private b = true;
  private q = WireState.float;
  constructor(simulator: Simulator, id: string, param: BaseComponentParam) {
    super(simulator, id, param);
    this.pins.set('a', new TriState(this, -25, 0));
    this.pins.set('b', new TriState(this, 0, -20));
    this.pins.set('q', new TriState(this, 25, 0));
  }

  setup(canvas: SVGElement) {
    super.doSetup('logicinvtristate', canvas);
  }

  io() {
    this.a = this.binary(this.getPin('a').getAndReset(), true);
    this.b = this.binary(this.getPin('b').getAndReset(), true);
  }

  update(): boolean {
    const oldstate = this.q;
    this.q = this.b ? WireState.float : this.a ? WireState.high : WireState.low;
    this.getPin('q').setState(this.q);
    return oldstate !== this.q;
  }
}
