// Momentary push button component for the LogicSimulator
// Copyright (C) 2022 Sascha Nitsch

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// eslint-disable-next-line @typescript-eslint/no-unused-vars
class MomentaryPushButton extends BaseComponent {
  private pressed = false;
  private a: WireState = WireState.float;
  private q: WireState = WireState.float;
  constructor(simulator: Simulator, id: string, param: BaseComponentParam) {
    super(simulator, id, param);
    this.pins.set('a', new TriState(this, 0, 0, undefined, { type: 'MomentaryPushButton', id: id + '-a' }));
    this.pins.set('q', new TriState(this, 0, 0, undefined, { type: 'MomentaryPushButton', id: id + '-q' }));
  }

  setup(canvas: SVGElement) {
    super.doSetup('momentarypushbutton', canvas);
    if (this.element) {
      this.element.addEventListener('mousedown', this.activate.bind(this));
    }
    document.addEventListener('mouseup', this.deactivate.bind(this));
  }

  activate(e: MouseEvent) {
    e.stopPropagation();
    this.pressed = true;
    this.simulator.manualtick();
  }

  deactivate(e: MouseEvent) {
    e.stopPropagation();
    if (this.pressed) {
      this.pressed = false;
      this.simulator.manualtick();
    }
  }

  io() {
    this.a = this.getPin('a').getAndReset();
  }

  update(): boolean {
    const oldstate = this.q;
    this.q = this.pressed ? this.a : WireState.float;
    this.getPin('q').setState(this.q);
    return oldstate !== this.q;
  }
}
